#ifndef QEMAILWRAPPER_H
#define QEMAILWRAPPER_H

/*
***************************************************************************
Copyright (C) 2000-2010 KBasic Software: www.kbasic.com. All rights reserved.
Source codes are copyrighted by Bernd Noetscher 2000-2010.

You may use this source codes under the terms of the GNU Public License (Version 3) as published 
by the Free Software Foundation. 

If you are interested in using the source codes for other licenses or 
commercial development, you must buy commercial licenses from KBasic Software.
***************************************************************************
*/


#include <QObject>
#include <QEventLoop>
#include <QFile>
#include <QDir>
#include <QPair>

#include "qxtmail_tiny/qxtmailmessage.h"
#include "qxtmail_tiny/qxtmailattachment.h"
#include "qxtmail_tiny/qxtsmtp.h"
#include "qxtmail_tiny/qxtpop3.h"

#define QXT_NETWORK_EXPORT

/*
    Methods are not thread-safe!
*/

class Q7BEmailWrapper : public QObject
{
    Q_OBJECT
    QxtSmtp * smtp ;
    QxtPop3 * pop3 ;
    QString smtpServerHost ;
    QString pop3ServerHost ;
    int smtpPort ;
    int pop3Port ;
    bool secured ;
    QString LastError ;

public:

    Q7BEmailWrapper() : QObject(){}

    /*
        smtpSrvrHst - smtp mail server host name
        smtpPrt - smtp mail server port
        pop3SrvrHst - pop3 mail server host name
        pop3Prt - pop3 mail server port
        scrd - if true then connection will be encrypted
        username - mail server login
        password - mail server password
    */
    void login( QString smtpSrvrHst ,
                            int smtpPrt ,
                            QString pop3SrvrHst ,
                            int pop3Prt ,
                            bool ssl_scrd ,
                            QString username ,
                            QString password);

    /*
        from - From field
        toRecipients - Recipients list
        toRecipientsCC - CC list
        toRecipientsBCC - BCC list
        subject - subject Field string
        body - body field string
        attachmenFilestList - absolute or relative files names list
        Method is synchronous, returns "" on success and error string on error
    */
    QString sendMail( QString from ,
                      QStringList toRecipients ,
                      QStringList toRecipientsCC ,
                      QStringList toRecipientsBCC ,
                      QString subject ,
                      QString body ,
                      QStringList attachmenFilestList ) ;

    /*
        folderForSavingFiles - method tries to write attachment files to this folder. if empty,
        filed will not saved.
        Method is synchronous, returns QStringList on success error string only, otherwise: 1st element subject, 2nd body
    */
    QStringList receiveNextMail( QString folderForSavingFiles ) ;

    /*
        Returns the number of available messages
        returns -1 on error
    */

    int listMailCount();

    /*
        which - number of message (range can be retrieved by listMail)
        folderForSavingFiles - method tries to write attachment files to this folder. if empty,
        filed will not saved.
        Method is synchronous, returns "" on success and error string on error
    */
    QString receiveMail( int which ,
                         QString & subject ,
                         QString & body ,
                       //  QList < QPair < QString , QByteArray > > * attachments ,
                         QString folderForSavingFiles ) ;
    /*
        Returns list of available messages numbers and sizes
        Method is synchronous, returns "" on success and error string on error
    */
    QString listMail( QList < QPair < int , int > > & attachments ) ;
signals:
    void stop() ;
public slots :
private slots :
    void smtpConnected() ;
    void smtpEncrypted() ;
    void smtpAuthenticated() ;
    void smtpError( QByteArray er ) ;
    void smtpMailFailed ( int mailID, int errorCode, const QByteArray & msg ) ;
    void smtpMailSent ( int mailID ) ;
    void smtpRecipientRejected ( int mailID, const QString & address, const QByteArray & msg ) ;
    void smtpFinished () ;
    void smtpSenderRejected ( int mailID, const QString & address, const QByteArray & msg ) ;

    void pop3Connected() ;
    void pop3Encrypted() ;
    void pop3Authenticated() ;
    void pop3Error( QByteArray er ) ;
    void pop3finished ( int code ) ;

private:
QList < QPair < int , int > > _msgInfoList ;

};



#endif // QEMAILWRAPPER_H
