#include "widgetmanager.h"
#include "selectionsquares.h"
#include "designingarea.h"
#include "formmaket.h"
#include "widgetfactory.h"

#include <QMetaObject>
#include <QMetaProperty>
#include <QClipboard>
#include <QApplication>
#include <QBuffer>
#include <QDataStream>
#include <QDebug>
#include "tabordereditwidget.h"
#include "constants.h"
#include "relativelayout.h"
#include "props_functions.h"
#include <QToolTip>

WidgetManager::WidgetManager (QObject *parent) :
    QObject (parent)
  , currentActiveArea ( 0 )
  , dragSrcArea ( 0 )
{
}

void WidgetManager::focusChanged ( DesigningArea * d )
{
    currentActiveArea = d ;
}

void WidgetManager::setDeleted ( bool b , QList < QPointer< QWidget > > list )
{
    if ( ! currentActiveArea )
        return ;
    for( int k = 0 ; k < list.count () ; k ++ )
    {
        if( qobject_cast < DesigningArea * > ( list [ k ]  ) )
            list.removeAt( k ) ;
    }
    bool f = false ;
    bool sc = false ;
    foreach ( QPointer< QWidget > s , list )
    {
        if ( s )
        {
            if( b )
            {
                WidgetFactory::instance()->removeChildFromWidget( s->parentWidget() , s ) ;
                s->setVisible ( false ) ;
            }
            else if ( WidgetFactory::instance()->addChildToWidget( s->parentWidget() , s ) )
                s->setVisible ( true ) ;
            s->setProperty( DELETED_PROP_NAME , b ) ;

            if ( b )
            {
                removeSelection ( s ) ;
                sc = true ;
            }
            f = true ;
        }
    }
    if ( f )
    {
        currentActiveArea->update () ;
        emit structureChanged () ;
        finishTabOrderEditing () ;
    }
    if( sc )
        emit selectionChanged () ;
    updateSectionMinimumSize ( 0 ) ;
}

void WidgetManager::moveWidgets ( QPoint delta , QList < QPointer< QWidget > > list )
{
    QSet< QLayout * > layoutsToUpdate ;

    foreach ( QPointer< QWidget > s , list )
        if ( s )
        {
            //INCORRECT, NEED TO USE WIDGETFACTORY

            if ( s->parentWidget()->layout() )
                layoutsToUpdate.insert( s->parentWidget()->layout() ) ;
            s->move ( s->pos () + delta ) ;
            updateRelGeomValues( s ) ;
        }
    foreach ( QLayout * l , layoutsToUpdate )
        l->update() ;

    tuneSelectionsGeometry () ;
    tuneDesigningAreaSize ( list ) ;
    emit selectionChanged () ;
    updateSectionMinimumSize ( 0 ) ;
    finishTabOrderEditing () ;

}

void WidgetManager::resizeWidgets ( QPoint delta, int direction , QList < QPointer< QWidget > > list)
{
    if ( list.isEmpty () )
        return ;
    QSet< QLayout * > layoutsToUpdate ;
    foreach ( QPointer< QWidget > s , list )
        if ( s )
        {
            //INCORRECT, NEED TO USE WIDGETFACTORY
            if ( s->parentWidget() && s->parentWidget()->layout() )
                layoutsToUpdate.insert( s->parentWidget()->layout() ) ;
            switch ( direction )
            {
            case 0 :
                s->setGeometry ( QRect ( s->geometry ().topLeft () + delta , QSize ( s->geometry ().width () - delta.x () , s->geometry ().height () - delta.y () ) ) ) ;
                break ;
            case 1 :
                s->setGeometry ( QRect ( s->geometry ().topLeft () + QPoint ( 0 , delta.y () ) , QSize ( s->geometry ().width () , s->geometry ().height () - delta.y () ) ) ) ;
                break ;
            case 2 :
                s->setGeometry ( QRect ( s->geometry ().topLeft () + QPoint ( 0 , delta.y () ) , QSize ( s->geometry ().width () + delta.x (), s->geometry ().height () - delta.y () ) ) ) ;
                break ;
            case 3 :
                s->setGeometry ( QRect ( s->geometry ().topLeft () , QSize ( s->geometry ().width () + delta.x () , s->geometry ().height () ) ) ) ;
                break ;
            case 4 :
                s->setGeometry ( QRect ( s->geometry ().topLeft () , s->geometry ().bottomRight () + delta ) ) ;
                break ;
            case 5 :
                s->setGeometry ( QRect ( s->geometry ().topLeft () , QSize ( s->geometry ().width () , s->geometry ().height () + delta.y () ) ) ) ;
                break ;
            case 6 :
                s->setGeometry ( QRect ( s->geometry ().topLeft () + QPoint ( delta.x () , 0 ) , QSize ( s->geometry ().width () - delta.x () , s->geometry ().height () + delta.y () ) ) ) ;
                break ;
            case 7 :
                s->setGeometry ( QRect ( s->geometry ().topLeft () + QPoint ( delta.x () , 0 ) , QSize ( s->geometry ().width () - delta.x () , s->geometry ().height () ) ) ) ;
                break ;
            }
            updateRelGeomValues( s ) ;

        }
    foreach ( QLayout * l , layoutsToUpdate )
        l->update() ;

    tuneSelectionsGeometry () ;

    tuneDesigningAreaSize ( list ) ;
    emit selectionChanged () ;
    updateSectionMinimumSize ( 0 ) ;
    finishTabOrderEditing () ;

}

void WidgetManager::applyPropertiesToWidgets ( QVariantMap props , QList < QPointer< QWidget > > list )
{
    QSet< QLayout * > layoutsToUpdate ;
    for ( int j = 0 ; j < list.count () ; j ++ )
    {

        QWidget * widget = list [ j ] ;
        if ( widget )
        {
            if ( widget->parentWidget()->layout() )
                layoutsToUpdate.insert( widget->parentWidget()->layout() ) ;
            QMap < QString , QVariant >::const_iterator i = props.constBegin () ;
            while  (i != props.constEnd () )
            {
                PF::writeProperty( widget , i.key() , i.value() ) ;

                ++ i ;
            }
            if ( ! ( props.keys().contains( REL_L )
                     || props.keys().contains( REL_R )
                     || props.keys().contains( REL_T )
                     || props.keys().contains( REL_B ) ) )
                updateRelGeomValues( widget ) ;
        }
    }

    foreach ( QLayout * l , layoutsToUpdate )
        l->update() ;

    tuneSelectionsGeometry () ;

    tuneDesigningAreaSize ( list ) ;
    emit structureChanged ();
    emit selectionChanged();
    updateSectionMinimumSize ( 0 ) ;
    finishTabOrderEditing () ;

}

void WidgetManager::updateRelGeomValues ( QWidget * s )
{
    //qDebug() << s->property( REL_R ).toInt() ;

    return ; // debug

    if ( s->property( EN_REL_L ).toBool() )
        s->setProperty( REL_L , s->geometry().left() ) ;
    if ( s->property( EN_REL_R ).toBool() )
        s->setProperty( REL_R , s->parentWidget()->width() - s->frameGeometry().right() ) ;
    if ( s->property( EN_REL_T ).toBool() )
        s->setProperty( REL_T , s->geometry().top() ) ;
    if ( s->property( EN_REL_B ).toBool() )
        s->setProperty( REL_B , s->parentWidget()->height() - s->geometry().bottom() ) ;
}

DesigningArea * WidgetManager::activeArea ()
{
    return currentActiveArea ;
}

QList<QPointer<QWidget> > WidgetManager::widgetsFromClbd ( QWidget * parent , QPoint offset )
{
    QList<QPointer<QWidget> > widgets ;

    QClipboard *clbd = QApplication::clipboard () ;
    const QMimeData *mime = clbd->mimeData () ;
    QByteArray serializedWidgets = mime->data ( "serialized/objects" ) ;

    QBuffer buf ;
    buf.setData ( serializedWidgets ) ;
    buf.open ( QIODevice::ReadOnly ) ;

    QDataStream stream ( & buf ) ;

    QVariantMap sectionWidgets ;
    stream >> sectionWidgets ;

    QMap < int , QPointer<QWidget> > zReorderMap ;

    for ( int j = 0 ; j < sectionWidgets.count () ; j ++ )
    {
        QList < QPointer<QWidget> > wl = WidgetFactory::instance()->restoreWidgets ( parent ,
                                                   sectionWidgets.values () [ j ].toMap () , currentActiveArea , true ) ;
        allWidgets.append ( wl ) ;
        foreach ( QWidget * li , wl )
        {
            RelativeLayout * l = qobject_cast < RelativeLayout * > (li->parentWidget()->layout() ) ;
            if ( l )
                connect( l , SIGNAL(widgetGeomUpdated(QWidget*)) , SLOT(widgetChangedSize(QWidget*)) , Qt::UniqueConnection) ;
        }
        //qDebug() << wl.first()->geometry() ;
        if ( ! wl.isEmpty() )
            if( wl.first()->parentWidget() )
                zReorderMap.insert( wl.first()->property( Z_ORDER ).toInt()
                                    , wl.first() ) ;  // root widget of tree, which is produced by restoreWidgets

        widgets.append ( wl ) ;
    }
    //qDebug() << "widgetsFromClbd" << zReorderMap ;

    if ( zReorderMap.count() > 1 )
    {
        QMap < int , QPointer<QWidget> >::iterator i = zReorderMap.end() - 1 ;
        while ( i != zReorderMap.begin() )
        {

            QPointer<QWidget> higher = i.value() ;
            //qDebug() << "higher: " << higher->objectName() ;
            -- i ;
            QPointer<QWidget> lower = i.value() ;
            //qDebug() << "lower: " << lower->objectName() ;
            lower->stackUnder( higher ) ;

        }
    }
    QRect commonEdge ;
    foreach ( QWidget * w , widgets )
        if ( w && w->parentWidget () == parent )
        {
            commonEdge = commonEdge.united ( w->geometry () ) ;
        }
    foreach ( QWidget * w , widgets )
    {
        if ( w && w->parentWidget () == parent )
        {
            w->move ( w->pos () - commonEdge.topLeft () + offset ) ;
        }
        w->setGeometry ( currentActiveArea->rectToGrid ( w->geometry () ) ) ;

        updateRelGeomValues ( w ) ;

        if ( w->parentWidget() )
            addSelection ( w ) ;

        //updateRecursive( w ) ;
    }
    emit selectionChanged () ;
    tuneDesigningAreaSize ( widgets ) ;
    emit structureChanged () ;
    updateSectionMinimumSize ( 0 ) ;
    finishTabOrderEditing () ;

    return widgets ;
}

void WidgetManager::tuneDesigningAreaSize ( QList<QPointer<QWidget> > list )
{
    int ws = - 1 ;
    int hs = - 1 ;
    DesigningArea * parentArea = 0 ;
    foreach ( QPointer< QWidget > s , list )
    {
        if ( s )
        {
            if ( ! parentArea )
                parentArea = findWidgetArea ( s ) ;
            if ( s->geometry ().right () - parentArea->width () > ws )
                ws = s->geometry ().right () - parentArea->width () ;
            if ( s->geometry ().bottom () - parentArea->height () > hs )
                hs = s->geometry ().bottom () - parentArea->height () ;
        }
    }
    //qDebug() << ws << hs ;
    if ( ws >= 0 || hs >= 0 )
        parentArea->emitGeomSmall ( parentArea->width () +  (  ( ws >= 0 ) ? ws + 10 : 0 )  ,
                                           parentArea->height () +  (  ( hs >= 0 ) ? hs + 10 : 0 ) )  ;
}

QList < QPointer < QWidget > > WidgetManager::getCurrentSelList ()
{
    QList < QPointer < QWidget > > ret ;
    foreach ( QPointer< SelectionSquares > s , getSelections () )
        if ( s )
            ret.append ( s->getTarget () )  ;
    if ( currentActiveArea )
    {
        if ( currentActiveArea->currentWidget ()
             && isWidgetSelected( currentActiveArea->currentWidget () ) )
        {
            ret.removeAll ( currentActiveArea->currentWidget () ) ;
            if ( currentActiveArea->currentWidget () )
                ret.prepend ( currentActiveArea->currentWidget () ) ;
        }
    }
    return ret ;
}

QList < QPointer < QWidget > > WidgetManager::getDragWidgets ()
{
    QList < QPointer < QWidget > > ret = draggedWidgets ;
    draggedWidgets.clear () ;
    return ret ;
}

void WidgetManager::widgetsDragged ( QWidget * w )
{
    w = WidgetFactory::instance()->findKnownParent( w ) ;
    if ( ! w )
        return ;
    if ( ! isWidgetSelected ( w ) )
    {
        clearSelection () ;
        addSelection ( w ) ;
        emit selectionChanged () ;
    }

    deleteSelDublicates () ;

    emit selectionChanged() ;
    draggedWidgets = getCurrentSelList () ;
    dragSrcArea = currentActiveArea ;

    copySelected ( true ) ;

    QPoint hotSpot = w->mapFromGlobal ( QCursor::pos () ) ;

    QMimeData *mimeData = new QMimeData;
    mimeData->setText (w->metaObject ()->className ());
    QDrag *drag = new QDrag ( dragSrcArea ) ;

    QRect commonEdge ;
    foreach ( QWidget * w , draggedWidgets )
        if ( w )
            commonEdge = commonEdge.united ( w->geometry () ) ;

    QPixmap pixmap ( commonEdge.size () ) ;
    pixmap.fill ( QColor ( 0 , 0 , 0 , 0 ) ) ;

    foreach ( QWidget * w , draggedWidgets )
        if ( w )
            w->render ( & pixmap , w->pos () - commonEdge.topLeft () ) ;

    drag->setHotSpot ( hotSpot + w->pos () - commonEdge.topLeft () ) ;
    drag->setPixmap (pixmap);
    mimeData->setData ("application/x-hotspot",
                       QByteArray::number (drag->hotSpot ().x ())
                       + " " + QByteArray::number (drag->hotSpot ().y ()));
    drag->setMimeData ( mimeData ) ;


    widgetsToRemove.clear () ;

    foreach ( QWidget * w , draggedWidgets )
        if ( w )
        {
            widgetsToRemove.append ( w ) ;
            WidgetFactory::instance()->removeChildFromWidget( w->parentWidget() , w ) ;
            w->setVisible ( false ) ;
            w->setProperty( DELETED_PROP_NAME , true ) ;
        }
    clearSelection () ;
    finishTabOrderEditing () ;
    updateSectionMinimumSize ( 0 ) ;
    QApplication::processEvents();
    if ( drag->exec ( /*Qt::CopyAction , Qt::CopyAction*/ ) == Qt::IgnoreAction ) // SOMETIMES CRASHES ON MAC
        foreach ( QPointer < QWidget > s , widgetsToRemove )
            if ( s )
            {

                if ( WidgetFactory::instance()->addChildToWidget ( s->parentWidget() , s ) )
                    s->setVisible ( true ) ;

                s->setProperty( DELETED_PROP_NAME , false ) ;
                widgetByName.insert ( s->objectName () , s ) ;
            }
}

DesigningArea * WidgetManager::getDragSrc ()
{
    return dragSrcArea ;
}

void WidgetManager::selectWidgets ( QList < QPointer < QWidget > > list, bool withControl )
{
    finishTabOrderEditing () ;
    if ( ! withControl )
    {
        clearSelection () ;
        foreach ( QPointer< QWidget > s , list )
            if ( s )
                addSelection ( s ) ;
        emit selectionChanged () ;
    }
    else
    {
        foreach ( QPointer< QWidget > s , list )
        {
            if ( isWidgetSelected ( s ) )
                removeSelection ( s ) ;
            else
                addSelection ( s ) ;
        }
        emit selectionChanged () ;
    }
}

void WidgetManager::selectAll ()
{
    clearSelection () ;
    foreach ( QPointer< QWidget > s , allWidgets )
        if ( s && ! s->property( DELETED_PROP_NAME ).toBool() )
            addSelection ( s ) ;
    emit selectionChanged () ;
}

void WidgetManager::clearSelection ()
{
    foreach ( QPointer< SelectionSquares > s , selections )
        if ( s ) delete s ;
    selections.clear () ;
    selByWidgetHash.clear () ;
    widgetBySelHash.clear () ;
    emit selectionChanged ( ) ;
}

void WidgetManager::addSelection ( QWidget * w )
{
    //qDebug() << w->metaObject()->className() ;
    if ( isWidgetSelected ( w ) || w->property( DELETED_PROP_NAME ).toBool())
        return ;
    if( ! qobject_cast < DesigningArea * > ( w )
            && ( ! ( w = WidgetFactory::instance()->findKnownParent( w ) ) ) )
        return ;

    if( w )
    {
        QPointer< SelectionSquares > s = new SelectionSquares ( w , w->parentWidget () ) ;
        s->setProperty ( SEL_SQUARES_MARK , true ) ;
        connect ( s.data () , SIGNAL (moveSignal (QPoint,int)) , SLOT (mouseResize (QPoint,int)) ) ;
        connect ( s.data () , SIGNAL (endMoveSignal ()) , SIGNAL (endResizing ()) ) ;
        QWidget * da = w->parentWidget () ;
        while ( ! qobject_cast < DesigningArea * > ( da ) && da != 0 )
            da = da->parentWidget () ;

        s->installEventFilter ( da ) ;
        selections.append ( s ) ;
        widgetBySelHash.insert ( s.data () , w ) ;
        selByWidgetHash.insert ( w , s.data () ) ;
        if(currentActiveArea)
            currentActiveArea->update () ;
        if(s->parentWidget ())
            s->parentWidget ()->update () ;
        //emit selectionChanged ( ) ;
    }
}

bool WidgetManager::isWidgetSelected ( QWidget * w )
{
    return selByWidgetHash.contains ( w ) ;
}

void WidgetManager::removeSelection ( QWidget * w )
{
    if ( selByWidgetHash.contains ( w ) )
    {
        SelectionSquares * s = selByWidgetHash.value ( w ) ;
        selections.removeAll ( s ) ;
        widgetBySelHash.remove ( s ) ;
        selByWidgetHash.remove ( w ) ;
        delete s ;

    }
    //emit selectionChanged ( ) ;
}

QList< QPointer < SelectionSquares > > WidgetManager::getSelections ()
{
    return selections ;
}

void WidgetManager::tuneSelectionsGeometry ()
{
    foreach ( QPointer< SelectionSquares > s , selections )
        if ( s ) s->tuneGeometry ();
}

void WidgetManager::copySelected ( bool clearId )
{
    QByteArray serializedWidgets ;

    if ( getSelections ().isEmpty () )
        return ;

    QList< QWidget * > toRemove ;
    QPoint delta ;
    QRect commonEdge ;

    foreach ( QPointer < SelectionSquares > s , getSelections () )
    {
        if ( s && s->getTarget () )
        {
            QWidget * p = s->getTarget ()->parentWidget () ;
            while ( p != 0 && ! qobject_cast < DesigningArea * >  ( p ) )
            {
                if ( isWidgetSelected ( p ) )
                    toRemove.append ( s->getTarget () ) ;
                p = p->parentWidget () ;
            }
            if ( toRemove.isEmpty () )
                commonEdge = commonEdge.united ( s->getTarget ()->geometry () ) ;
        }
    }

    delta = commonEdge.topLeft () ;

    foreach ( QWidget * w , toRemove )
        removeSelection ( w ) ;
    emit selectionChanged () ;

    QVariantMap sectionWidgets ;

    foreach ( QPointer < SelectionSquares > s , getSelections () )
    {
        if ( s )
        {
            QVariantMap map = WidgetFactory::instance()->createChildPropList ( s->getTarget () , delta , clearId ) ;
            sectionWidgets.insertMulti ( "widget" , map ) ;
        }
    }

    QBuffer buf ;
    buf.open ( QIODevice::WriteOnly ) ;
    QDataStream stream ( & buf ) ;
    stream << sectionWidgets ;
    serializedWidgets = buf.data () ;

    QMimeData *mimeDataClbd = new QMimeData ;

    mimeDataClbd->setData ( "serialized/objects" , serializedWidgets ) ;
    QClipboard *clbd = QApplication::clipboard () ;
    clbd->clear () ;
    clbd->setMimeData ( mimeDataClbd ) ;
}

void WidgetManager::mouseResize ( QPoint delta , int i )
{
    pointMod += delta ;

    int xD = pointMod.x() / activeArea ()->getGridDelta() * activeArea ()->getGridDelta() ;
    int yD = pointMod.y() / activeArea ()->getGridDelta() * activeArea ()->getGridDelta() ;
    if ( xD != 0 || yD != 0 )
    {
        emit resizeWidgets ( QPoint( xD , yD ) , i ) ;
        pointMod.setX ( pointMod.x() % activeArea ()->getGridDelta() ) ;
        pointMod.setY ( pointMod.y() % activeArea ()->getGridDelta() ) ;
    }


    finishTabOrderEditing () ;

}

QByteArray WidgetManager::save ( FormMaket * maket )
{
    QList < QPointer < QWidget > > del ;
    QVariantMap all ;
    foreach ( DesigningArea *  ar , maket->getAreas () )
    {
        QVariantMap sectionWidgets ;
        foreach ( QObject * ob , ar->children () )
        {
            QWidget * w = qobject_cast < QWidget * > ( ob ) ;
            if ( w && ! qobject_cast< SelectionSquares * > ( w ) && ! w->property( DELETED_PROP_NAME ).toBool() )
            {
                QVariantMap map = WidgetFactory::instance()->createChildPropList ( w , QPoint() , false ) ;

                sectionWidgets.insertMulti ( "widget" , map ) ;
                del.append ( w ) ;
            }
        }
        all.insert ( maket->nameOfSection ( ar ) , sectionWidgets ) ;
    }
    QVariantMap sectionSizes ;
    foreach ( DesigningArea *  ar , maket->getAreas () )
        sectionSizes.insert ( ar->objectName () , ar->size () ) ;
    all.insert ( "section sizes" , sectionSizes ) ;
    QBuffer buf ;
    buf.open ( QIODevice::WriteOnly ) ;
    QDataStream stream ( & buf ) ;
    stream << all ;
    return buf.data () ;
}

void WidgetManager::load ( FormMaket * maket , QByteArray data )
{
    QVariantMap all ;
    QBuffer buf ;
    buf.setData ( data ) ;
    buf.open ( QIODevice::ReadOnly ) ;

    QDataStream stream ( & buf ) ;
    stream >> all ;

    QVariantMap sectionSizes = all.value ( "section sizes"  ).toMap ()  ;
    foreach ( DesigningArea *  ar , maket->getAreas () )
        ar->setGeometry ( QRect ( QPoint ( 0 , 0 ) , sectionSizes.value ( ar->objectName () ).toSize () ) ) ;

    QList < QPointer < QWidget > > newWidgets ;
    for ( int i = 0 ; i < all.count () ; i ++ )
    {
        DesigningArea * a = maket->sectionByName ( all.keys () [ i ] ) ;
        if ( a )
        {
            currentActiveArea = a ;
            QVariantMap sectionWidgets  = all.values () [ i ].toMap () ;
            //QStringList z_orderedNames = sectionWidgets.value( "ZOrderList" ).toStringList() ;
            // qDebug() << z_orderedNames ;
            //sectionWidgets.remove( "ZOrderList" ) ;

            QMap < int , QPointer<QWidget> > zReorderMap ;
            for ( int j = 0 ; j < sectionWidgets.count () ; j ++ )
            {
                QList<QPointer<QWidget> > wl = WidgetFactory::instance()->restoreWidgets ( a ,
                                                      sectionWidgets.values () [ j ].toMap () , currentActiveArea , false ) ;
                if ( ! wl.isEmpty() )
                    if( wl.first()->parentWidget() )
                        zReorderMap.insert( wl.first()->property( Z_ORDER ).toInt()
                                            , wl.first() ) ;  // root widget of tree, which is produced by restoreWidgets
                newWidgets += wl ;
                foreach ( QWidget * li , wl )
                {
                    RelativeLayout * l = qobject_cast < RelativeLayout * > (li->parentWidget()->layout() ) ;
                    if ( l )
                        connect( l , SIGNAL(widgetGeomUpdated(QWidget*)) , SLOT(widgetChangedSize(QWidget*)) , Qt::UniqueConnection) ;
                }
            }
            //qDebug() << "load: " << zReorderMap ;
            if ( zReorderMap.count() > 1 )
            {
                QMap < int , QPointer<QWidget> >::iterator i = zReorderMap.end() - 1 ;
                while ( i != zReorderMap.begin() )
                {


                    QPointer<QWidget> higher = i.value() ;
                    //qDebug() << "higher: " << higher->objectName() ;
                    -- i ;
                    QPointer<QWidget> lower = i.value() ;
                    //qDebug() << "lower: " << lower->objectName() ;
                    lower->stackUnder( higher ) ;

                }
            }
            updateSectionMinimumSize ( 0 ) ;
        }
    }

    /*
        tab order restoration
    */
    QList < QWidget * > orderedWidgets ;
    for( int i = 1 ; i <= newWidgets.count () ; i ++ )
        foreach ( QWidget * w , newWidgets )
            if( w->property( TABINDEX_PROP_NAME ).toInt()  == i )
            {
                orderedWidgets.append ( w ) ;
                break ;
            }
    for( int i = 0 ; i < orderedWidgets.count () - 1 ; i ++ )
        QWidget::setTabOrder( orderedWidgets [ i ] , orderedWidgets [ i + 1 ] ) ;

    foreach ( QWidget * w , newWidgets )
        updateRelGeomValues ( w ) ;

    emit structureChanged () ;
    tuneSelectionsGeometry () ;
    finishTabOrderEditing () ;

}


void WidgetManager::widgetChangedSize( QWidget * w )
{
    SelectionSquares * s = selByWidgetHash.value( w ) ;
    if ( s )
        s->tuneGeometry() ;
}

void WidgetManager::alignWidgets ( QList < QPointer < QWidget > > list , AligningSide side )
{
    switch ( side )
    {
    case Top :
        if ( ! list.isEmpty () )
        {
            int m = list [ 0 ]->geometry ().top () ;
            foreach ( QWidget * w , list )
                if ( w && w->geometry ().top () < m )
                    m = w->geometry ().top () ;
            foreach ( QWidget * w , list )
            {
                if ( w )
                {
                    w->move ( w->pos ().x () , m ) ;
                    w->setGeometry ( currentActiveArea->rectToGrid ( w->geometry () ) ) ;
                }
            }
            tuneSelectionsGeometry () ;
        }
        break ;
    case Right :
        if ( ! list.isEmpty () )
        {
            int m = list [ 0 ]->geometry ().right () ;
            foreach ( QWidget * w , list )
                if ( w && w->geometry ().right () > m )
                    m = w->geometry ().right () ;
            foreach ( QWidget * w , list )
            {
                if ( w )
                {
                    w->move ( m - w->width () , w->pos ().y () ) ;
                    w->setGeometry ( currentActiveArea->rectToGrid ( w->geometry () ) ) ;
                }
            }
            tuneSelectionsGeometry () ;
        }
        break ;
    case Bottom :
        if ( ! list.isEmpty () )
        {
            int m = list [ 0 ]->geometry ().bottom () ;
            foreach ( QWidget * w , list )
                if ( w && w->geometry ().bottom () > m )
                    m = w->geometry ().bottom () ;
            foreach ( QWidget * w , list )
            {
                if ( w )
                {
                    w->move ( w->pos ().x () , m - w->height () ) ;
                    w->setGeometry ( currentActiveArea->rectToGrid ( w->geometry () ) ) ;
                }
            }
            tuneSelectionsGeometry () ;
        }
        break ;
    case Left :
    {
        if ( ! list.isEmpty () )
        {
            int m = list [ 0 ]->geometry ().left () ;
            foreach ( QWidget * w , list )
                if ( w && w->geometry ().left () < m )
                    m = w->geometry ().left () ;
            foreach ( QWidget * w , list )
            {
                if ( w )
                {
                    w->move ( m , w->pos ().y () ) ;
                    w->setGeometry ( currentActiveArea->rectToGrid ( w->geometry () ) ) ;
                }
            }
            tuneSelectionsGeometry () ;
        }
        break ;
    }
    }
    updateSectionMinimumSize ( 0 ) ;
    finishTabOrderEditing () ;

}

void WidgetManager::setWidgetPos ( QList < QPointer < QWidget > > list , QPoint pos )
{
    foreach ( QWidget * w , list )
        if ( w )
        {
            w->move ( pos ) ;
            w->setGeometry ( currentActiveArea->rectToGrid ( w->geometry () ) ) ;
            updateRelGeomValues( w ) ;
        }
    tuneSelectionsGeometry () ;
    updateSectionMinimumSize ( 0 ) ;
    finishTabOrderEditing () ;

}

void WidgetManager::updateSectionMinimumSize (  DesigningArea *   )
{
    emit updateSectionSize ( 0 ) ;
}

void WidgetManager::copyProperties ()
{
    if( ! getCurrentSelList ().isEmpty() )
    {
        propertiesBuffer.clear() ;
        QWidget * w = getCurrentSelList () [ 0 ]  ;
        for ( int i = 0 ; i < PF::metaPropertyCount ( w ) ; i ++ )
            if( PF::checkMetaPropertyWritable( w , i ) )
                propertiesBuffer.insert ( PF::metaPropertyName( w , i ) , PF::readMetaProperty( w , i ) ) ;

        int dpCount = PF::visibleDynPropsCount ( w ) ;
        for( int i = 0 ; i < dpCount ; i ++ )
        {
            QString n = PF::getVisibleDynPropName ( i , w ) ;
            propertiesBuffer.insert( n , w->property ( n.toAscii().constData() ) ) ;
        }
    }
    propertiesBuffer.remove( "pos" ) ;
    propertiesBuffer.remove( "objectName" ) ;
    propertiesBuffer.remove( "geometry" ) ;
}

void WidgetManager::startTabOrderEditing ()
{
    finishTabOrderEditing () ;

    foreach ( QWidget * w , allWidgets )
    {
        if ( w
             && w->dynamicPropertyNames().contains( DELETED_PROP_NAME )
             && WidgetFactory::instance()->isClassSupported ( w->metaObject()->className() )
             && ! w->property( DELETED_PROP_NAME ).toBool() )
        {
            DesigningArea * parentArea = findWidgetArea ( w ) ;
            TabOrderEditWidget * tw = new TabOrderEditWidget ( w , parentArea ) ;
            tw->move ( parentArea->mapFromGlobal ( w->mapToGlobal ( QPoint ( w->width () * 0.382 , w->height () / 2 ) ) ) -
                       QPoint ( tw->width () / 2 , tw->height () / 2 ) ) ;
            tw->show () ;
            tw->raise () ;
        }
    }
}

void WidgetManager::finishTabOrderEditing ()
{
    //clearSelection() ;
    TabOrderEditWidget::finishEditing () ;
}

DesigningArea * WidgetManager::findWidgetArea ( QWidget * w )
{
    if( w )
    {
        QWidget * p = w->parentWidget() ;
        while( qobject_cast< DesigningArea * > ( p ) == 0 && p != 0 )
            p = p->parentWidget() ;
        return qobject_cast< DesigningArea * > ( p ) ;
    }
    return 0 ;
}


QVariantMap WidgetManager::getPropBuffer ()
{
    return propertiesBuffer ;
}

QList < QPointer < QWidget > > WidgetManager::morphWidgetsInto ( QList < QPointer < QWidget > > list , QString className )
{
    QList < QPointer < QWidget > > ret ;
    for( int k = 0 ; k < list.count () ; k ++ )
    {
        if( qobject_cast < DesigningArea * > ( list [ k ]  ) )
            list.removeAt( k ) ;
    }

    for( int k = 0 ; k < list.count () ; k ++ )
    {

        QPointer < QWidget > newW = WidgetFactory::instance()->newWidgetByClassName( className , "" , true ) ;

        if( newW )
        {
            allWidgets.append ( newW ) ;
            WidgetFactory::instance ()->addChildToWidget ( list [ k ]->parentWidget () , newW ) ;
            //newW->installEventFilter ( currentActiveArea ) ;
            WidgetFactory::instance ()->addEventFilterToWidget ( currentActiveArea , newW ) ;
            for( int i = 0 ; i < PF::metaPropertyCount ( list [ k ] ) ; i ++ )
            {
                PF::writeMetaProperty( newW.data()
                                       , PF::metaPropertyIndex( newW.data() , PF::metaPropertyName( list [ k ] , i ) )
                                       , PF::readMetaProperty( list [ k ].data() , i )) ;

            }
            foreach ( QString dpName , list [ k ]->dynamicPropertyNames () )
            {
                if ( ! PF::checkPropIsInternal ( dpName ) )
                    if ( newW->dynamicPropertyNames ().contains ( dpName.toAscii() ) )
                        newW->setProperty ( dpName.toAscii().constData() , list [ k ]->property( dpName.toAscii().constData() ) ) ;
            }
            ret.append( newW )  ;
        }
    }
    setDeleted( true , list ) ;
    emit structureChanged () ;
    updateSectionMinimumSize ( 0 ) ;
    return ret ;
}
void WidgetManager::makeGeometryUnRelated ()
{
    QList < QPointer < QWidget > > list = getCurrentSelList () ;
    clearSelection() ;
    foreach( QWidget * w , list )
    {

        if ( w && w->parentWidget () )
        {
            addSelection ( w ) ;
            //            double relX = double ( w->x() ) / double ( w->parentWidget()->width () ) ;
            //            double relY = double ( w->y() ) / double ( w->parentWidget()->height () ) ;
            //            double relW = double ( w->width() ) / double ( w->parentWidget()->width () ) ;
            //            double relH = double ( w->height() ) / double ( w->parentWidget()->height () ) ;
            QVariantMap m ;
            //            m.insert ( EN_REL_X , false ) ;
            //            m.insert ( EN_REL_Y , false ) ;
            //            m.insert ( EN_REL_W , false ) ;
            //            m.insert ( EN_REL_H , false ) ;

            m.insert ( EN_REL_L , false ) ;
            m.insert ( EN_REL_R , false ) ;
            m.insert ( EN_REL_T , false ) ;
            m.insert ( EN_REL_B , false ) ;
            emit propsEdited ( m ) ;
            clearSelection () ;
        }
    }
    foreach( QWidget * w , list )
        addSelection( w ) ;
    emit selectionChanged () ;
}

void WidgetManager::makeGeometryRelated ()
{
    QList < QPointer < QWidget > > list = getCurrentSelList () ;
    clearSelection() ;
    foreach( QWidget * w , list )
    {

        if ( w && w->parentWidget () )
        {
            addSelection ( w ) ;
            QVariantMap m ;

            m.insert ( REL_L , w->geometry().left() ) ;
            m.insert ( REL_R , w->parentWidget()->width() - w->geometry().right() ) ;
            m.insert ( REL_T , w->geometry().top() ) ;
            m.insert ( REL_B , w->parentWidget()->height() - w->geometry().bottom() ) ;
            m.insert ( EN_REL_L , true ) ;
            m.insert ( EN_REL_R , true ) ;
            m.insert ( EN_REL_T , true ) ;
            m.insert ( EN_REL_B , true ) ;
            emit propsEdited ( m ) ;
            clearSelection () ;
        }
    }
    foreach( QWidget * w , list )
        addSelection( w ) ;
    emit selectionChanged () ;
}

void WidgetManager::moveToBack ()
{

    QList < QPointer < QWidget > > list = getCurrentSelList () ;
    if( ! list.isEmpty () )
    {
        QWidget * w = list.first () ;
        if ( w )
        {
            w->lower () ;
            w->setProperty( Z_ORDER , w->parentWidget()->children().indexOf( w ) ) ;

            tuneSelectionsGeometry () ;
        }
    }
}

void WidgetManager::moveToFront ()
{
    QList < QPointer < QWidget > > list = getCurrentSelList () ;
    if( ! list.isEmpty () )
    {
        QWidget * w = list.first () ;
        if ( w )
        {
            //qDebug() << PF::groupedProperties( w ) ;

            w->raise () ;
            w->setProperty( Z_ORDER , w->parentWidget()->children().indexOf( w ) ) ;
            tuneSelectionsGeometry () ;
        }
    }
}

void WidgetManager::setSameHeight()
{
    QList < QPointer < QWidget > > list = getCurrentSelList () ;
    int maxHeight = 0 ;
    foreach ( QWidget * w , list )
        if ( w )
            if ( w->height() > maxHeight )
                maxHeight = w->height() ;

    clearSelection() ;
    foreach( QWidget * w , list )
    {
        QVariantMap m ;
        m.insert ( "size" , QSize( w->width() , maxHeight ) ) ;
        addSelection ( w ) ;
        emit propsEdited ( m ) ;
        clearSelection () ;
    }
    foreach ( QWidget * w , list )
        addSelection( w ) ;
    emit selectionChanged () ;

}

void WidgetManager::setSameWidth()
{
    QList < QPointer < QWidget > > list = getCurrentSelList () ;
    int maxWidth = 0 ;
    foreach ( QWidget * w , list )
        if ( w )
            if ( w->width() > maxWidth )
                maxWidth = w->width() ;
    clearSelection() ;
    foreach( QWidget * w , list )
    {
        QVariantMap m ;
        m.insert ( "size" , QSize(maxWidth , w->height() ) ) ;
        addSelection ( w ) ;
        emit propsEdited ( m ) ;
        clearSelection () ;
    }
    foreach( QWidget * w , list )
        addSelection( w ) ;
    emit selectionChanged () ;

}

void WidgetManager::setSameSize()
{
    QList < QPointer < QWidget > > list = getCurrentSelList () ;
    QSize maxSize ;
    foreach ( QWidget * w , list )
        if ( w )
            if ( w->size().width() * w->size().height() > maxSize.width() * maxSize.height() )
                maxSize = w->size() ;
    QVariantMap m ;
    m.insert ( "size" , maxSize ) ;
    emit propsEdited ( m ) ;
}

void WidgetManager::applyPropertiesReplacements ( FormMaket * m )
{
    foreach ( DesigningArea *  ar , m->getAreas () )
    {
        foreach ( QObject * ob , ar->children () )
        {
            QWidget * w = qobject_cast < QWidget * > ( ob ) ;
            if ( w && ! qobject_cast< SelectionSquares * > ( w ) && ! w->property( DELETED_PROP_NAME ).toBool() )
                replToRealProp ( w ) ;

        }
    }
}

void WidgetManager::replToRealProp ( QWidget * w )
{
    for ( int i = 0 ; i < PF::metaPropertyCount ( w ) ; i ++ )
    {
        if ( PF::hasReplacement( w , PF::metaPropertyName( w ,i ) ) )
            PF::writeMetaPropertyD( w , PF::metaPropertyName( w ,i ) ,
                                    PF::readProperty( w , PF::metaPropertyName( w ,i ) )) ;
    }

    //    foreach( QByteArray pn , w->dynamicPropertyNames() )
    //        props.insert ( QString( pn ) ,
    //                      w->property( pn.data() ) ) ;


    foreach ( QWidget * child , WidgetFactory::instance ()->getChildren ( w ) )
    {
        if ( child && ! child->property( DELETED_PROP_NAME ).toBool() )
            replToRealProp ( child ) ;
    }
}

void WidgetManager::setNoLayout()
{
    QList < QPointer < QWidget > > list = getCurrentSelList () ;
    if( ! list.isEmpty () )
    {
        QWidget * w = list.first () ;
        setLayoutOfWidget( w , new RelativeLayout( w ) ) ;
    }
}

void WidgetManager::setHorLayout()
{
    QList < QPointer < QWidget > > list = getCurrentSelList () ;
    if( ! list.isEmpty () )
    {
        QWidget * w = list.first () ;
        setLayoutOfWidget( w , new QHBoxLayout( w ) ) ;
    }
}

void WidgetManager::setVertLayout()
{
    QList < QPointer < QWidget > > list = getCurrentSelList () ;
    if( ! list.isEmpty () )
    {
        QWidget * w = list.first () ;
        setLayoutOfWidget( w , new QVBoxLayout( w ) ) ;
    }
}

void WidgetManager::setLayoutOfWidget ( QWidget * w , QLayout * l )
{
    if ( w )
    {
        QList < QPointer < QWidget > > wl =  WidgetFactory::instance ()->getChildren ( w ) ;
        if ( ! wl.isEmpty() )
        {
            if ( wl.first()->parentWidget() )
            {
                if ( wl.first()->parentWidget()->layout( ) )
                    delete  wl.first()->parentWidget()->layout( ) ;
                wl.first()->parentWidget()->setLayout( l ) ;
                foreach( QWidget * child , wl )
                    if( child )
                        wl.first()->parentWidget()->layout()->addWidget( child ) ;
                wl.first()->parentWidget()->layout()->update();
                tuneSelectionsGeometry () ;
                return ;
            }
        }
    }
    delete l ;
}

void WidgetManager::showTooltipForWidget ( QWidget * w )
{
    if ( WidgetFactory::instance()->isClassSupported ( w->metaObject()->className() ) )
        QToolTip::showText ( QCursor::pos()
                             , QString ( "%1 As %2" ).arg( w->objectName() ).arg( w->metaObject()->className() )
                             , w ) ;
}

void WidgetManager::deleteSelDublicates()
{
    QList< QWidget * > toRemove ;

    foreach ( QWidget * w , getCurrentSelList () )
        if ( w )
        {
            widgetByName.remove ( w->objectName () ) ;
            QWidget * p = w->parentWidget () ;
            if ( qobject_cast < DesigningArea * >  ( w ) )
                toRemove.append ( w ) ;
            else
                while ( p != 0 && ! qobject_cast < DesigningArea * >  ( p ) )
                {
                    widgetByName.remove ( p->objectName () ) ;
                    if ( isWidgetSelected ( p ) )
                        toRemove.append ( w ) ;
                    p = p->parentWidget () ;
                }
        }

    foreach ( QWidget * w , toRemove )
        removeSelection ( w ) ;
}
