/*
    This file containes declaration of WidgetManager class,
    which provides methods for viewing and editing QWidget objects
*/

#ifndef WIDGETMANAGER_H
#define WIDGETMANAGER_H

#include <QObject>
#include <QPointer>
#include <QHash>
#include <QPoint>
#include <QVariantMap>


//Manages widgets, its selections, changing its position, size, etc.
//Provides methods for copy/paste, drag&drop.
class DesigningArea ;
class FormMaket ;
class SelectionSquares ;
class TabOrderEditWidget ;
class QLayout ;
class WidgetManager : public QObject
{
    Q_OBJECT
    //Pointer to DesigningArea, which is in focus now.
    QPointer < DesigningArea > currentActiveArea ;
    //Pointer to DesigningArea, from which dragging has been started.
    DesigningArea * dragSrcArea ;
    //List of widgets, which user tries to drag
    QList < QPointer < QWidget > > draggedWidgets ;
    //List off all widgets
    QList < QPointer < QWidget > > allWidgets ;
    //Hash, which allows to find widget by object name
    QHash < QString , QWidget * > widgetByName ;
    //Pointer to maket, which are handled by WidgetManager
    FormMaket * maket ;

    QHash < QWidget * , DesigningArea * > widgetsParents ;
    //List of SelectionSquares objects, which stores current selection
    QList < QPointer < SelectionSquares > > selections ;

    QHash< SelectionSquares * , QWidget * > widgetBySelHash ;
    QHash< QWidget * , SelectionSquares * > selByWidgetHash ;

    QPoint pointMod ;

    //Seems like the same as draggedWidgets...
    QList < QPointer < QWidget > > widgetsToRemove ;

    void updateSectionMinimumSize ( DesigningArea * ar ) ;
    // Plays a role of copy/paste buffer for properties editing
    QVariantMap propertiesBuffer ;
    void updateRelGeomValues ( QWidget * s ) ;

    void setLayoutOfWidget ( QWidget * w , QLayout * l ) ;
public:
    explicit WidgetManager (QObject *parent = 0);
    DesigningArea * activeArea () ;

    //creates widgets, which were previosly stored in clipboard by copySelected or by WPalette class.
    QList < QPointer < QWidget > > widgetsFromClbd ( QWidget *parent, QPoint hs) ;
    //Returns list of current selected widgets
    QList < QPointer < QWidget > > getCurrentSelList () ;
    //Returns list of widget, which user tries to drag
    QList < QPointer < QWidget > > getDragWidgets () ;
    //Returns DesigningArea, from which dragging has been started
    DesigningArea * getDragSrc () ;
    //returns SelectionSquares objects list, which stores current selection
    QList< QPointer < SelectionSquares > > getSelections () ;
    //Converts current maket state to QByteArray
    QByteArray save ( FormMaket * maket ) ;
    //Fills maket from QByteArray
    void load ( FormMaket * maket , QByteArray data ) ;
    //Checks if widget is selected.
    bool isWidgetSelected ( QWidget * w ) ;
    enum AligningSide
    {
        Top ,
        Right ,
        Bottom ,
        Left
    } ;
    QVariantMap getPropBuffer () ;
    // Returns Designing area, which containes
    DesigningArea * findWidgetArea( QWidget * w ) ;
    // Creates widgets of class "className" and copies properties of widgets in list to them
    QList < QPointer < QWidget > > morphWidgetsInto ( QList < QPointer < QWidget > > list , QString className ) ;
    void applyPropertiesReplacements ( FormMaket * m ) ;
    void replToRealProp ( QWidget * w ) ;
signals:
    // Emitted when something changes in selection
    void selectionChanged ( ) ;
    ///need to process geomSmall situation somewhere here
    ///
    void resizeWidgets ( QPoint delta , int i ) ;
    void endResizing () ;
    //emitted when widgets hierarhy changes
    void structureChanged () ;
    void updateSectionSize ( DesigningArea * ) ;
    // called when properties of selected widgets must be changed
    void propsEdited ( QVariantMap v ) ;
private slots :
    void mouseResize ( QPoint delta , int i ) ;
    // Updates selection for widget, if widget is selected
    void widgetChangedSize( QWidget * w ) ;
public slots :
    //changes currentActiveArea
    void focusChanged ( DesigningArea * ) ;
    //marks widgets deleted or undeleted
    void setDeleted (bool b, QList<QPointer<QWidget> > list ) ;
    //moves widgets from list to delta
    void moveWidgets ( QPoint delta  , QList < QPointer< QWidget > > list ) ;
    //resizes widgets in selected direction
    void resizeWidgets ( QPoint delta,int direction, QList < QPointer< QWidget > > list ) ;
    //assigns properties to widgets
    void applyPropertiesToWidgets ( QVariantMap props , QList < QPointer< QWidget > > list ) ;
    //Checks if designing area is too small.
    void tuneDesigningAreaSize ( QList<QPointer<QWidget> > list ) ;
    //Starts dragging process for widgets on current DesigningArea
    void widgetsDragged ( QWidget * ) ;
    //selects all widget on entire maket
    void selectAll () ;
    //selects widgets
    void selectWidgets ( QList < QPointer < QWidget > > list , bool withControl ) ;
    //cancels selection
    void clearSelection () ;
    //makes widget selected
    void addSelection ( QWidget * w ) ;
    //makes widget unselected
    void removeSelection ( QWidget * ) ;
    void tuneSelectionsGeometry () ;
    //updates selection geometry
    void copySelected ( bool clearId = false ) ;
    // Moves widgets for aligning
    void alignWidgets ( QList < QPointer < QWidget > > list , AligningSide side ) ;
    // Moves widgets to pos
    void setWidgetPos ( QList < QPointer < QWidget > > list , QPoint pos ) ;
    // Copies properties of first selected widget to propertiesBuffer
    void copyProperties() ;
    // Creates TabOrderEditWidget objects for all current widgets
    void startTabOrderEditing() ;
    // Finishes tab-order editing process
    void finishTabOrderEditing() ;

    void makeGeometryRelated () ;
    void makeGeometryUnRelated () ;

    void moveToBack () ;
    void moveToFront () ;

    void setSameHeight() ;
    void setSameWidth() ;
    void setSameSize() ;

    void setNoLayout() ;
    void setHorLayout() ;
    void setVertLayout() ;
    void showTooltipForWidget ( QWidget * w ) ;

    void deleteSelDublicates () ;

} ;

#endif // WIDGETMANAGER_H
